@@ -35,34 +35,27 @@
 #include <glib/gstdio.h>
 #include <glib/gi18n-lib.h>
 
-#include "libedataserver/e-sexp.h"
-#include "libedataserver/e-data-server-util.h"
-#include "libedataserver/e-flag.h"
-#include "libedataserver/e-url.h"
-#include "libebook/e-contact.h"
-#include "libebook/e-destination.h"
-#include "libedata-book/e-book-backend-sexp.h"
-#include "libedata-book/e-data-book.h"
-#include "libedata-book/e-data-book-view.h"
-#include "e-book-backend-ews.h"
-#include "e-book-backend-sqlitedb.h"
+/*#include <libedata-book/libedata-book.h>*/
+
 #include "lzx/ews-oal-decompress.h"
-#include "ews-oab-decoder.h"
 
 #include "server/e-ews-item-change.h"
 #include "server/e-ews-message.h"
 #include "server/e-ews-connection.h"
 #include "server/e-ews-item.h"
+#include "server/e-source-ews-folder.h"
 
 #include "utils/e-ews-query-to-restriction.h"
 
+#include "e-book-backend-ews.h"
+#include "ews-book-backend-sqlitedb.h"
+#include "ews-oab-decoder.h"
+
 #define d(x) x
 
 #define EDB_ERROR(_code) e_data_book_create_error (E_DATA_BOOK_STATUS_ ## _code, NULL)
 #define EDB_ERROR_EX(_code,_msg) e_data_book_create_error (E_DATA_BOOK_STATUS_ ## _code, _msg)
 
-G_DEFINE_TYPE (EBookBackendEws, e_book_backend_ews, E_TYPE_BOOK_BACKEND)
-
 static gboolean ebews_fetch_items (EBookBackendEws *ebews,  GSList *items, gboolean store_to_cache, GSList **vcards, GCancellable *cancellable, GError **error);
 
 typedef struct {
@@ -80,7 +73,7 @@ struct _EBookBackendEwsPrivate {
 	gchar *username;
 	gchar *password;
 
-	EBookBackendSqliteDB *ebsdb;
+	EwsBookBackendSqliteDB *ebsdb;
 
 	gboolean only_if_exists;
 	gboolean is_writable;
@@ -97,7 +90,6 @@ struct _EBookBackendEwsPrivate {
 	GThread *dthread;
 	SyncDelta *dlock;
 
-	ECredentials *credentials;
 	GCancellable *cancellable;
 };
 
@@ -121,6 +113,47 @@ enum {
 #define PRIV_LOCK(p)   (g_static_rec_mutex_lock (&(p)->rec_mutex))
 #define PRIV_UNLOCK(p) (g_static_rec_mutex_unlock (&(p)->rec_mutex))
 
+/* Forward Declarations */
+static void	e_book_backend_ews_authenticator_init
+				(ESourceAuthenticatorInterface *interface);
+
+G_DEFINE_TYPE_WITH_CODE (
+	EBookBackendEws,
+	e_book_backend_ews,
+	E_TYPE_BOOK_BACKEND,
+	G_IMPLEMENT_INTERFACE (
+		E_TYPE_SOURCE_AUTHENTICATOR,
+		e_book_backend_ews_authenticator_init))
+
+static CamelEwsSettings *
+book_backend_ews_get_collection_settings (EBookBackendEws *backend)
+{
+	ESource *source;
+	ESource *collection;
+	ESourceCamel *extension;
+	ESourceRegistry *registry;
+	CamelSettings *settings;
+	const gchar *extension_name;
+
+	source = e_backend_get_source (E_BACKEND (backend));
+	registry = e_book_backend_get_registry (E_BOOK_BACKEND (backend));
+
+	extension_name = e_source_camel_get_extension_name ("ews");
+	e_source_camel_generate_subtype ("ews", CAMEL_TYPE_EWS_SETTINGS);
+
+	/* The collection settings live in our parent data source. */
+	collection = e_source_registry_find_extension (
+		registry, source, extension_name);
+	g_return_val_if_fail (collection != NULL, NULL);
+
+	extension = e_source_get_extension (collection, extension_name);
+	settings = e_source_camel_get_settings (extension);
+
+	g_object_unref (collection);
+
+	return CAMEL_EWS_SETTINGS (settings);
+}
+
 static void
 convert_error_to_edb_error (GError **perror)
 {
@@ -158,12 +191,6 @@ convert_error_to_edb_error (GError **perror)
 	*perror = error;
 }
 
-static void
-ews_auth_required (EBookBackend *backend)
-{
-	e_book_backend_notify_auth_required (backend, TRUE, NULL);
-}
-
 static gboolean
 ews_remove_attachments (const gchar *attachment_dir)
 {
@@ -898,6 +925,8 @@ ews_create_contact_cb (GObject *object,
 	/* get a list of ids from server (single item) */
 	e_ews_connection_create_items_finish (cnc, res, &items, &error);
 
+	g_return_if_fail (ebews->priv->ebsdb != NULL);
+
 	if (error == NULL) {
 		EEwsItem *item = (EEwsItem *) items->data;
 
@@ -906,7 +935,7 @@ ews_create_contact_cb (GObject *object,
 
 		e_contact_set (create_contact->contact, E_CONTACT_UID, item_id->id);
 		e_contact_set (create_contact->contact, E_CONTACT_REV, item_id->change_key);
-		e_book_backend_sqlitedb_add_contact (ebews->priv->ebsdb, ebews->priv->folder_id, create_contact->contact, FALSE, &error);
+		ews_book_backend_sqlitedb_add_contact (ebews->priv->ebsdb, ebews->priv->folder_id, create_contact->contact, FALSE, &error);
 
 		if (error == NULL) {
 			GSList *contacts;
@@ -1023,8 +1052,10 @@ ews_book_remove_contact_cb (GObject *object,
 
 	simple = G_SIMPLE_ASYNC_RESULT (res);
 
+	g_return_if_fail (priv->ebsdb != NULL);
+
 	if (!g_simple_async_result_propagate_error (simple, &error))
-		deleted = e_book_backend_sqlitedb_remove_contacts (priv->ebsdb, priv->folder_id, remove_contact->sl_ids, &error);
+		deleted = ews_book_backend_sqlitedb_remove_contacts (priv->ebsdb, priv->folder_id, remove_contact->sl_ids, &error);
 
 	if (deleted)
 		e_data_book_respond_remove_contacts (remove_contact->book, remove_contact->opid, EDB_ERROR (SUCCESS),  remove_contact->sl_ids);
@@ -1123,6 +1154,8 @@ ews_modify_contact_cb (GObject *object,
 
 	e_ews_connection_update_items_finish (cnc, res, &items, &error);
 
+	g_return_if_fail (priv->ebsdb != NULL);
+
 	if (error == NULL) {
 		EEwsItem *item = (EEwsItem *) items->data;
 
@@ -1134,8 +1167,8 @@ ews_modify_contact_cb (GObject *object,
 
 		id = e_contact_get (modify_contact->old_contact, E_CONTACT_UID);
 
-		e_book_backend_sqlitedb_remove_contact (priv->ebsdb, priv->folder_id, id, &error);
-		e_book_backend_sqlitedb_add_contact (ebews->priv->ebsdb, ebews->priv->folder_id, modify_contact->new_contact, FALSE, &error);
+		ews_book_backend_sqlitedb_remove_contact (priv->ebsdb, priv->folder_id, id, &error);
+		ews_book_backend_sqlitedb_add_contact (ebews->priv->ebsdb, ebews->priv->folder_id, modify_contact->new_contact, FALSE, &error);
 
 		if (error == NULL) {
 			GSList *new_contacts;
@@ -1249,6 +1282,8 @@ e_book_backend_ews_modify_contacts (EBookBackend *backend,
 		return;
 	}
 
+	g_return_if_fail (priv->ebsdb != NULL);
+
 	contact = e_contact_new_from_vcard (vcards->data);
 
 	id = g_new0 (EwsId, 1);
@@ -1263,7 +1298,7 @@ e_book_backend_ews_modify_contacts (EBookBackend *backend,
 		return;
 	}
 
-	old_contact = e_book_backend_sqlitedb_get_contact ( priv->ebsdb, priv->folder_id,
+	old_contact = ews_book_backend_sqlitedb_get_contact ( priv->ebsdb, priv->folder_id,
 				 id->id, NULL, NULL, &error);
 	if (!old_contact) {
 		g_object_unref (contact);
@@ -1328,14 +1363,14 @@ e_book_backend_ews_get_contact_list (EBookBackend *backend,
 	priv = ebews->priv;
 
 	if (!e_backend_get_online (E_BACKEND (backend))) {
-		if (e_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL)) {
-			list = e_book_backend_sqlitedb_search (priv->ebsdb, priv->folder_id, query, NULL, NULL, NULL, &error);
+		if (priv->ebsdb && ews_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL)) {
+			list = ews_book_backend_sqlitedb_search (priv->ebsdb, priv->folder_id, query, NULL, NULL, NULL, &error);
 			l = list;
 			while (l) {
-				EbSdbSearchData *s_data = (EbSdbSearchData *) l->data;
+				EwsSdbSearchData *s_data = (EwsSdbSearchData *) l->data;
 
 				vcard_list = g_slist_append (vcard_list, g_strdup (s_data->vcard));
-				e_book_backend_sqlitedb_search_data_free (s_data);
+				ews_book_backend_sqlitedb_search_data_free (s_data);
 				l = l->next;
 			}
 			convert_error_to_edb_error (&error);
@@ -1355,14 +1390,14 @@ e_book_backend_ews_get_contact_list (EBookBackend *backend,
 		return;
 	}
 
-	if (e_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL)) {
-		list = e_book_backend_sqlitedb_search (priv->ebsdb, priv->folder_id, query, NULL, NULL, NULL, &error);
+	if (priv->ebsdb && ews_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL)) {
+		list = ews_book_backend_sqlitedb_search (priv->ebsdb, priv->folder_id, query, NULL, NULL, NULL, &error);
 		l = list;
 		while (l) {
-			EbSdbSearchData *s_data = (EbSdbSearchData *) l->data;
+			EwsSdbSearchData *s_data = (EwsSdbSearchData *) l->data;
 
 			vcard_list = g_slist_append (vcard_list, g_strdup (s_data->vcard));
-			e_book_backend_sqlitedb_search_data_free (s_data);
+			ews_book_backend_sqlitedb_search_data_free (s_data);
 			l = l->next;
 		}
 
@@ -1619,7 +1654,10 @@ ews_gal_needs_update (EBookBackendEws *cbews,
 	gboolean ret = FALSE;
 	gchar *tmp;
 
-	tmp = e_book_backend_sqlitedb_get_key_value (priv->ebsdb, priv->folder_id, "seq", error);
+	if (!priv->ebsdb)
+		return FALSE;
+
+	tmp = ews_book_backend_sqlitedb_get_key_value (priv->ebsdb, priv->folder_id, "seq", error);
 	if (error)
 		goto exit;
 
@@ -1685,12 +1723,16 @@ ews_remove_old_gal_file (EBookBackendEws *cbews,
 	EBookBackendEwsPrivate *priv = cbews->priv;
 	gchar *filename;
 
-	filename = e_book_backend_sqlitedb_get_key_value (priv->ebsdb, priv->folder_id, "oab-filename", error);
+	if (!priv->ebsdb)
+		return FALSE;
+
+	filename = ews_book_backend_sqlitedb_get_key_value (priv->ebsdb, priv->folder_id, "oab-filename", error);
 	if (*error)
 		return FALSE;
 
 	if (filename)
 		g_unlink (filename);
+	g_free (filename);
 
 	return TRUE;
 }
@@ -1711,6 +1753,8 @@ ews_gal_store_contact (EContact *contact,
 	struct _db_data *data = (struct _db_data *) user_data;
 	EBookBackendEwsPrivate *priv = data->cbews->priv;
 
+	g_return_if_fail (priv->ebsdb != NULL);
+
 	data->contact_collector = g_slist_prepend (data->contact_collector, g_object_ref (contact));
 	data->collected_length += 1;
 
@@ -1725,7 +1769,7 @@ ews_gal_store_contact (EContact *contact,
 		g_free (status_message);
 
 		data->contact_collector = g_slist_reverse (data->contact_collector);
-		e_book_backend_sqlitedb_add_contacts (priv->ebsdb, priv->folder_id, data->contact_collector, FALSE, error);
+		ews_book_backend_sqlitedb_add_contacts (priv->ebsdb, priv->folder_id, data->contact_collector, FALSE, error);
 
 		for (l = data->contact_collector; l != NULL; l = g_slist_next (l))
 			e_book_backend_notify_update (E_BOOK_BACKEND (data->cbews), E_CONTACT (l->data));
@@ -1751,12 +1795,14 @@ ews_replace_gal_in_db (EBookBackendEws *cbews,
 	gboolean ret = TRUE;
 	struct _db_data data;
 
+	g_return_val_if_fail (priv->ebsdb != NULL, FALSE);
+
 	/* remove the old address-book and create a new one in db */
-	if (e_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL)) {
-		ret = e_book_backend_sqlitedb_delete_addressbook (priv->ebsdb, priv->folder_id, error);
+	if (ews_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL)) {
+		ret = ews_book_backend_sqlitedb_delete_addressbook (priv->ebsdb, priv->folder_id, error);
 		ews_remove_attachments (priv->attachment_dir);
 		if (ret)
-			ret = e_book_backend_sqlitedb_create_addressbook (priv->ebsdb, priv->folder_id, priv->folder_name, TRUE, error);
+			ret = ews_book_backend_sqlitedb_create_addressbook (priv->ebsdb, priv->folder_id, priv->folder_name, TRUE, error);
 	}
 
 	if (!ret)
@@ -1775,7 +1821,7 @@ ews_replace_gal_in_db (EBookBackendEws *cbews,
 	       return ret;
 
 	/* mark the db as populated */
-	ret = e_book_backend_sqlitedb_set_is_populated (priv->ebsdb, priv->folder_id, TRUE, error);
+	ret = ews_book_backend_sqlitedb_set_is_populated (priv->ebsdb, priv->folder_id, TRUE, error);
 
 	return ret;
 }
@@ -1806,9 +1852,16 @@ ebews_start_gal_sync (gpointer data)
 		goto exit;
 	}
 
+	g_warn_if_fail (priv->ebsdb != NULL);
+	if (!priv->ebsdb)
+		goto exit;
+
+	if (full_l == NULL)
+		goto exit;
+
 	full = (EwsOALDetails *) full_l->data;
 	/* TODO fetch differential updates if available instead of downloading the whole GAL */
-	if (!e_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL) || ews_gal_needs_update (cbews, full, &error)) {
+	if (!ews_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL) || ews_gal_needs_update (cbews, full, &error)) {
 		gchar *seq;
 
 		d(printf ("Ewsgal: Downloading full gal \n");)
@@ -1831,11 +1884,11 @@ ebews_start_gal_sync (gpointer data)
 			goto exit;
 
 		seq = g_strdup_printf ("%"G_GUINT32_FORMAT, full->seq);
-		ret = e_book_backend_sqlitedb_set_key_value (priv->ebsdb, priv->folder_id, "seq", seq, &error);
+		ret = ews_book_backend_sqlitedb_set_key_value (priv->ebsdb, priv->folder_id, "seq", seq, &error);
 		g_free (seq);
 
 		if (!ret) {
-			e_book_backend_sqlitedb_delete_addressbook (priv->ebsdb, priv->folder_id, &error);
+			ews_book_backend_sqlitedb_delete_addressbook (priv->ebsdb, priv->folder_id, &error);
 			goto exit;
 		}
 	}
@@ -1883,12 +1936,14 @@ ebews_sync_deleted_items (EBookBackendEws *ebews,
 
 	priv = ebews->priv;
 
+	g_return_if_fail (priv->ebsdb != NULL);
+
 	for (l = deleted_ids; l != NULL; l = g_slist_next (l)) {
 		gchar *id = (gchar *) l->data;
 		gboolean partial_content;
 
-		if (e_book_backend_sqlitedb_has_contact (priv->ebsdb, priv->folder_id, id, &partial_content, NULL))
-			e_book_backend_sqlitedb_remove_contact (priv->ebsdb, priv->folder_id, id, error);
+		if (ews_book_backend_sqlitedb_has_contact (priv->ebsdb, priv->folder_id, id, &partial_content, NULL))
+			ews_book_backend_sqlitedb_remove_contact (priv->ebsdb, priv->folder_id, id, error);
 		e_book_backend_notify_remove (E_BOOK_BACKEND (ebews), id);
 	}
 
@@ -1907,6 +1962,8 @@ ebews_store_contact_items (EBookBackendEws *ebews,
 
 	priv = ebews->priv;
 
+	g_return_if_fail (priv->ebsdb != NULL);
+
 	for (l = new_items; l != NULL; l = g_slist_next (l)) {
 		EContact *contact;
 		gint i, element_type;
@@ -1936,7 +1993,7 @@ ebews_store_contact_items (EBookBackendEws *ebews,
 			/* store display_name, fileas, item id */	
 		}
 
-		e_book_backend_sqlitedb_add_contact (priv->ebsdb, priv->folder_id, contact, FALSE, error);
+		ews_book_backend_sqlitedb_add_contact (priv->ebsdb, priv->folder_id, contact, FALSE, error);
 		e_book_backend_notify_update (E_BOOK_BACKEND (ebews), contact);
 
 		g_object_unref (item);
@@ -2011,6 +2068,8 @@ ebews_store_distribution_list_items (EBookBackendEws *ebews,
 	GSList *l;
 	EContact *contact;
 
+	g_return_if_fail (ebews->priv->ebsdb != NULL);
+
 	contact = e_contact_new ();
 	e_contact_set (contact, E_CONTACT_UID, id->id);
 	e_contact_set (contact, E_CONTACT_REV, id->change_key);
@@ -2043,7 +2102,7 @@ ebews_store_distribution_list_items (EBookBackendEws *ebews,
 	}
 
 	g_slist_free (members);
-	e_book_backend_sqlitedb_add_contact (ebews->priv->ebsdb, ebews->priv->folder_id, contact, FALSE, error);
+	ews_book_backend_sqlitedb_add_contact (ebews->priv->ebsdb, ebews->priv->folder_id, contact, FALSE, error);
 	e_book_backend_notify_update (E_BOOK_BACKEND (ebews), contact);
 
 	g_object_unref (contact);
@@ -2225,11 +2284,13 @@ ebews_start_sync (gpointer data)
 	ebews = (EBookBackendEws *) data;
 	priv = ebews->priv;
 
+	g_return_val_if_fail (priv->ebsdb != NULL, FALSE);
+
 	status_message = g_strdup (_("Syncing contacts..."));
 	e_book_backend_foreach_view (E_BOOK_BACKEND (ebews), book_view_notify_status, status_message);
 	g_free (status_message);
 
-	sync_state = e_book_backend_sqlitedb_get_sync_data (priv->ebsdb, priv->folder_id, NULL);
+	sync_state = ews_book_backend_sqlitedb_get_sync_data (priv->ebsdb, priv->folder_id, NULL);
 	do
 	{
 		GSList *items_created = NULL, *items_updated = NULL;
@@ -2267,11 +2328,11 @@ ebews_start_sync (gpointer data)
 		if (error)
 			break;
 
-		e_book_backend_sqlitedb_set_sync_data (priv->ebsdb, priv->folder_id, sync_state, &error);
+		ews_book_backend_sqlitedb_set_sync_data (priv->ebsdb, priv->folder_id, sync_state, &error);
 	} while (!error && !includes_last_item);
 
 	if (!error)
-		e_book_backend_sqlitedb_set_is_populated (priv->ebsdb, priv->folder_id, TRUE, &error);
+		ews_book_backend_sqlitedb_set_is_populated (priv->ebsdb, priv->folder_id, TRUE, &error);
 
 	g_free (sync_state);
 
@@ -2358,7 +2419,7 @@ ebews_start_refreshing (EBookBackendEws *ebews)
 	PRIV_LOCK (priv);
 
 	if (e_backend_get_online (E_BACKEND (ebews)) &&
-	    priv->cnc != NULL&& priv->marked_for_offline)
+	    priv->cnc != NULL && priv->marked_for_offline)
 		fetch_deltas (ebews);
 
 	PRIV_UNLOCK (priv);
@@ -2381,15 +2442,17 @@ fetch_from_offline (EBookBackendEws *ews,
 		return;
 	}
 
-	contacts = e_book_backend_sqlitedb_search (priv->ebsdb, priv->folder_id, query, NULL, NULL, NULL, &error);
+	g_return_if_fail (priv->ebsdb != NULL);
+
+	contacts = ews_book_backend_sqlitedb_search (priv->ebsdb, priv->folder_id, query, NULL, NULL, NULL, &error);
 	for (l = contacts; l != NULL; l = g_slist_next (l)) {
-		EbSdbSearchData *s_data = (EbSdbSearchData *) l->data;
+		EwsSdbSearchData *s_data = (EwsSdbSearchData *) l->data;
 
 		/* reset vcard to NULL as it would be free'ed in prefiltered_vcard function */
 		e_data_book_view_notify_update_prefiltered_vcard (book_view, s_data->uid, s_data->vcard);
 		s_data->vcard = NULL;
 
-		e_book_backend_sqlitedb_search_data_free (s_data);
+		ews_book_backend_sqlitedb_search_data_free (s_data);
 	}
 
 	if (contacts)
@@ -2410,19 +2473,25 @@ e_book_backend_ews_start_book_view (EBookBackend *backend,
 	GCancellable *cancellable;
 	GSList *ids = NULL, *mailboxes = NULL, *l, *contacts = NULL, *c;
 	EwsFolderId *fid;
+	ESource *source;
+	ESourceRegistry *registry;
+	ESourceEwsFolder *extension;
+	const gchar *extension_name;
 	GError *error = NULL;
 	gboolean includes_last_item;
-	ESource *source;
 
 	ebews = E_BOOK_BACKEND_EWS (backend);
 	priv = ebews->priv;
 	query = e_data_book_view_get_card_query (book_view);
 
+	registry = e_book_backend_get_registry (backend);
+	source = e_backend_get_source (E_BACKEND (backend));
+
 	e_data_book_view_ref (book_view);
 	e_data_book_view_notify_progress (book_view, -1, _("Searching..."));
 
 	if (!e_backend_get_online (E_BACKEND (backend))) {
-		if (priv->ebsdb && e_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL)) {
+		if (priv->ebsdb && ews_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL)) {
 			fetch_from_offline (ebews, book_view, query, error);
 			return;
 		}
@@ -2434,19 +2503,27 @@ e_book_backend_ews_start_book_view (EBookBackend *backend,
 		return;
 	}
 
-	if (!priv->cnc) {
-		error = EDB_ERROR (AUTHENTICATION_REQUIRED);
-		ews_auth_required (backend);
-		e_data_book_view_notify_complete (book_view, error);
-		e_data_book_view_unref (book_view);
-		g_error_free (error);
-		return;
+	if (priv->cnc == NULL) {
+		/* XXX Why doesn't start_book_view()
+		 *     get passed a GCancellable? */
+		e_source_registry_authenticate_sync (
+			registry, source,
+			E_SOURCE_AUTHENTICATOR (backend),
+			NULL, &error);
+		if (error != NULL) {
+			e_data_book_view_notify_complete (book_view, error);
+			e_data_book_view_unref (book_view);
+			g_error_free (error);
+			return;
+		}
 	}
 
+	g_return_if_fail (priv->cnc != NULL);
+
 	ebews_start_refreshing (ebews);
 
 	if (priv->ebsdb &&
-	    e_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL)) {
+	    ews_book_backend_sqlitedb_get_is_populated (priv->ebsdb, priv->folder_id, NULL)) {
 		fetch_from_offline (ebews, book_view, query, error);
 		return;
 	}
@@ -2459,7 +2536,9 @@ e_book_backend_ews_start_book_view (EBookBackend *backend,
 		return;
 	}
 
-	source = e_backend_get_source (E_BACKEND (backend));
+	extension_name = E_SOURCE_EXTENSION_EWS_FOLDER;
+	extension = e_source_get_extension (source, extension_name);
+
 	cancellable = g_cancellable_new ();
 
 	/* FIXME Need to convert the Ids from EwsLegacyId format to EwsId format using
@@ -2467,7 +2546,7 @@ e_book_backend_ews_start_book_view (EBookBackend *backend,
 	 * 2007 and 2007_SP1 */
 	fid = g_new0 (EwsFolderId, 1);
 	fid->id = g_strdup (priv->folder_id);
-	fid->change_key = e_source_get_duped_property (source, "change-key");
+	fid->change_key = e_source_ews_folder_dup_change_key (extension);
 	ids = g_slist_append (ids, fid);
 
 	/* We do not scan until we reach the last_item as it might be good enough to show first 100
@@ -2560,58 +2639,64 @@ e_book_backend_ews_load_source (EBookBackend *backend,
 {
 	EBookBackendEws *cbews;
 	EBookBackendEwsPrivate *priv;
+	CamelEwsSettings *settings;
+	ESourceExtension *extension;
 	const gchar *cache_dir, *email;
-	const gchar *folder_name;
-	const gchar *offline, *is_gal;
-	GError *err = NULL;
+	const gchar *display_name;
+	const gchar *extension_name;
+	const gchar *gal_uid;
+	const gchar *uid;
 
 	cbews = E_BOOK_BACKEND_EWS (backend);
 	priv = cbews->priv;
 
 	cache_dir = e_book_backend_get_cache_dir (backend);
-	email = e_source_get_property (source, "email");
-	is_gal = e_source_get_property (source, "gal");
+	settings = book_backend_ews_get_collection_settings (cbews);
+	email = camel_ews_settings_get_email (settings);
 
-	if (is_gal && !strcmp (is_gal, "1"))
-		priv->is_gal = TRUE;
+	uid = e_source_get_uid (source);
+	gal_uid = camel_ews_settings_get_gal_uid (settings);
+	priv->is_gal = (g_strcmp0 (uid, gal_uid) == 0);
+
+	display_name = e_source_get_display_name (source);
+
+	extension_name = E_SOURCE_EXTENSION_EWS_FOLDER;
+	extension = e_source_get_extension (source, extension_name);
+
+	priv->folder_id = e_source_ews_folder_dup_id (
+		E_SOURCE_EWS_FOLDER (extension));
+
+	priv->ebsdb = ews_book_backend_sqlitedb_new (
+		cache_dir, email, priv->folder_id,
+		display_name, TRUE, perror);
+
+	if (priv->ebsdb == NULL)
+		return;
+
+	priv->marked_for_offline = FALSE;
+	priv->is_writable = FALSE;
 
 	if (!priv->is_gal) {
-		priv->folder_id = e_source_get_duped_property (source, "folder-id");
-		folder_name = e_source_peek_name (source);
+		extension_name = E_SOURCE_EXTENSION_OFFLINE;
+		extension = e_source_get_extension (source, extension_name);
 
-		priv->ebsdb = e_book_backend_sqlitedb_new (cache_dir, email, priv->folder_id, folder_name, TRUE, &err);
-		if (err) {
-			g_propagate_error (perror, err);
-			return;
-		}
+		priv->marked_for_offline =
+			e_source_offline_get_stay_synchronized (
+			E_SOURCE_OFFLINE (extension));
 
-		offline = e_source_get_property (source, "offline_sync");
-		if (offline  && g_str_equal (offline, "1"))
-			priv->marked_for_offline = TRUE;
-	} else {
-		priv->folder_id = e_source_get_duped_property (source, "oal_id");
+	/* If folder_id is present it means the GAL is marked for
+	 * offline usage, we do not check for offline_sync property */
+	} else if (priv->folder_id != NULL) {
+		priv->folder_name = g_strdup (display_name);
+		priv->oab_url = camel_ews_settings_dup_oaburl (settings);
 
-		/* If folder_id is present it means the GAL is marked for offline usage, we do not check for offline_sync property */
-		if (priv->folder_id) {
-			priv->folder_name = g_strdup (e_source_peek_name (source));
-			priv->oab_url = e_source_get_duped_property (source, "oab_url");
+		/* setup stagging dir, remove any old files from there */
+		priv->attachment_dir = g_build_filename (
+			cache_dir, "attachments", NULL);
+		g_mkdir_with_parents (priv->attachment_dir, 0777);
 
-			/* setup stagging dir, remove any old files from there */
-			priv->attachment_dir = g_build_filename (cache_dir, "attachments", NULL);
-			g_mkdir_with_parents (priv->attachment_dir, 0777);
-
-			priv->ebsdb = e_book_backend_sqlitedb_new (cache_dir, email, priv->folder_id, priv->folder_name, TRUE, &err);
-			if (err) {
-				g_propagate_error (perror, err);
-				return;
-			}
-			priv->marked_for_offline = TRUE;
-			priv->is_writable = FALSE;
-		}
+		priv->marked_for_offline = TRUE;
 	}
-
-	if (e_backend_get_online (E_BACKEND (backend)))
-		e_book_backend_notify_auth_required (backend, TRUE, NULL);
 }
 
 static void
@@ -2623,80 +2708,6 @@ e_book_backend_ews_remove (EBookBackend *backend,
 	e_data_book_respond_remove (book,  opid, EDB_ERROR (SUCCESS));
 }
 
-static void
-e_book_backend_ews_authenticate_user (EBookBackend *backend,
-                                      GCancellable *cancellable,
-                                      ECredentials *credentials)
-{
-	EBookBackendEws *ebews;
-	EBookBackendEwsPrivate *priv;
-	EEwsConnection *cnc = NULL;
-	ESource *esource;
-	GError *error = NULL;
-	GSList *folders = NULL, *ids = NULL;
-	EwsFolderId *fid = NULL;
-
-	const gchar *host_url;
-	const gchar *read_only;
-
-	ebews = E_BOOK_BACKEND_EWS (backend);
-	priv = ebews->priv;
-
-	if (!e_backend_get_online (E_BACKEND (backend))) {
-		e_book_backend_notify_opened (backend, EDB_ERROR (SUCCESS));
-		return;
-	}
-
-	if (priv->cnc) {
-		e_book_backend_notify_opened (backend, EDB_ERROR (SUCCESS));
-		return;
-	}
-
-	esource = e_backend_get_source (E_BACKEND (backend));
-	host_url = e_source_get_property (esource, "hosturl");
-	read_only = e_source_get_property (esource, "read_only");
-
-	cnc = e_ews_connection_new (host_url, e_credentials_peek (credentials, E_CREDENTIALS_KEY_USERNAME),
-					  e_credentials_peek (credentials, E_CREDENTIALS_KEY_PASSWORD),
-					  NULL, NULL, &error);
-
-	if ((read_only && !strcmp (read_only, "true")) || priv->is_gal) {
-		priv->is_writable = FALSE;
-	} else
-		priv->is_writable = TRUE;
-
-	if (!error && cnc) {
-		/* a dummy request to make sure we have a authenticated connection */
-		fid = g_new0 (EwsFolderId, 1);
-		fid->id = g_strdup ("contacts");
-		fid->is_distinguished_id = TRUE;
-		ids = g_slist_append (ids, fid);
-		e_ews_connection_get_folder_sync (
-			cnc, EWS_PRIORITY_MEDIUM, "Default",
-			NULL, ids, &folders, cancellable, &error);
-
-		e_ews_folder_free_fid (fid);
-		g_slist_free (ids);
-		ids = NULL;
-	}
-
-	if (error) {
-		convert_error_to_edb_error (&error);
-
-		e_book_backend_notify_auth_required (backend, TRUE, priv->credentials);
-		e_book_backend_notify_opened (backend, error);
-		g_object_unref (cnc);
-		return;
-	}
-
-	priv->cnc = cnc;
-	priv->username = e_source_get_duped_property (esource, "username");
-	priv->password = g_strdup (e_credentials_peek (credentials, E_CREDENTIALS_KEY_PASSWORD));
-
-	e_book_backend_notify_opened (backend, EDB_ERROR (SUCCESS));
-	e_book_backend_notify_readonly (backend, !priv->is_writable);
-}
-
 static void
 e_book_backend_ews_notify_online_cb (EBookBackend *backend,
                                      GParamSpec *spec)
@@ -2724,7 +2735,6 @@ e_book_backend_ews_notify_online_cb (EBookBackend *backend,
 
 			e_book_backend_notify_readonly (backend, !ebews->priv->is_writable);
 			e_book_backend_notify_online (backend, TRUE);
-			e_book_backend_notify_auth_required (backend, TRUE, NULL);
 		}
 	}
 }
@@ -2798,13 +2808,35 @@ e_book_backend_ews_open (EBookBackend *backend,
                          GCancellable *cancellable,
                          gboolean only_if_exists)
 {
-	GError *error = NULL;
+	EBookBackendEws *cbews;
+	ESourceRegistry *registry;
 	ESource *source;
+	GError *error = NULL;
 
+	cbews = E_BOOK_BACKEND_EWS (backend);
+
+	registry = e_book_backend_get_registry (backend);
 	source = e_backend_get_source (E_BACKEND (backend));
 	e_book_backend_ews_load_source (backend, source, only_if_exists, &error);
+
+	if (error == NULL) {
+		gboolean need_to_authenticate;
+
+		PRIV_LOCK (cbews->priv);
+		need_to_authenticate =
+			(cbews->priv->cnc == NULL) &&
+			(e_backend_get_online (E_BACKEND (backend)));
+		PRIV_UNLOCK (cbews->priv);
+
+		if (need_to_authenticate)
+			e_source_registry_authenticate_sync (
+				registry, source,
+				E_SOURCE_AUTHENTICATOR (backend),
+				cancellable, &error);
+	}
+
 	convert_error_to_edb_error (&error);
-	e_data_book_respond_open (book, opid, error);
+	e_book_backend_respond_opened (backend, book, opid, error);
 }
 
 /**
@@ -2891,9 +2923,6 @@ e_book_backend_ews_dispose (GObject *object)
 		priv->ebsdb = NULL;
 	}
 
-	e_credentials_free (priv->credentials);
-	priv->credentials = NULL;
-
 	g_static_rec_mutex_free (&priv->rec_mutex);
 
 	g_free (priv);
@@ -2902,6 +2931,98 @@ e_book_backend_ews_dispose (GObject *object)
 	G_OBJECT_CLASS (e_book_backend_ews_parent_class)->dispose (object);
 }
 
+static ESourceAuthenticationResult
+book_backend_ews_try_password_sync (ESourceAuthenticator *authenticator,
+                                    const GString *password,
+                                    GCancellable *cancellable,
+                                    GError **error)
+{
+	EBookBackendEws *backend;
+	EEwsConnection *connection;
+	ESourceAuthenticationResult result;
+	CamelEwsSettings *ews_settings;
+	CamelNetworkSettings *network_settings;
+	EwsFolderId *fid = NULL;
+	GSList *folders = NULL;
+	GSList *ids = NULL;
+	gchar *hosturl;
+	gchar *user;
+	GError *local_error = NULL;
+
+	/* This tests the password by fetching the contacts folder. */
+
+	backend = E_BOOK_BACKEND_EWS (authenticator);
+	ews_settings = book_backend_ews_get_collection_settings (backend);
+	hosturl = camel_ews_settings_dup_hosturl (ews_settings);
+
+	network_settings = CAMEL_NETWORK_SETTINGS (ews_settings);
+	user = camel_network_settings_dup_user (network_settings);
+
+	connection = e_ews_connection_new (
+		hosturl, user, password->str, NULL, NULL, error);
+
+	if (connection == NULL) {
+		result = E_SOURCE_AUTHENTICATION_ERROR;
+		goto exit;
+	}
+
+	fid = g_new0 (EwsFolderId, 1);
+	fid->id = g_strdup ("contacts");
+	fid->is_distinguished_id = TRUE;
+	ids = g_slist_append (ids, fid);
+
+	e_ews_connection_get_folder_sync (
+		connection, EWS_PRIORITY_MEDIUM, "Default",
+		NULL, ids, &folders, cancellable, &local_error);
+
+	e_ews_folder_free_fid (fid);
+	g_slist_free (ids);
+	ids = NULL;
+
+	if (local_error == NULL) {
+
+		PRIV_LOCK (backend->priv);
+
+		if (backend->priv->cnc != NULL)
+			g_object_unref (backend->priv->cnc);
+		backend->priv->cnc = g_object_ref (connection);
+
+		/* Stash the username and password for later
+		 * reuse in Offline Address Book connections. */
+		g_free (backend->priv->username);
+		g_free (backend->priv->password);
+		backend->priv->username = g_strdup (user);
+		backend->priv->password = g_strdup (password->str);
+
+		PRIV_UNLOCK (backend->priv);
+
+		result = E_SOURCE_AUTHENTICATION_ACCEPTED;
+
+	} else {
+		gboolean auth_failed;
+
+		auth_failed = g_error_matches (
+			local_error, EWS_CONNECTION_ERROR,
+			EWS_CONNECTION_ERROR_AUTHENTICATION_FAILED);
+
+		if (auth_failed) {
+			g_clear_error (&local_error);
+			result = E_SOURCE_AUTHENTICATION_REJECTED;
+		} else {
+			g_propagate_error (error, local_error);
+			result = E_SOURCE_AUTHENTICATION_ERROR;
+		}
+	}
+
+	g_object_unref (connection);
+
+exit:
+	g_free (hosturl);
+	g_free (user);
+
+	return result;
+}
+
 static void
 e_book_backend_ews_class_init (EBookBackendEwsClass *klass)
 {
@@ -2921,13 +3042,18 @@ e_book_backend_ews_class_init (EBookBackendEwsClass *klass)
 	parent_class->get_contact             = e_book_backend_ews_get_contact;
 	parent_class->get_contact_list        = e_book_backend_ews_get_contact_list;
 	parent_class->remove                  = e_book_backend_ews_remove;
-	parent_class->authenticate_user       = e_book_backend_ews_authenticate_user;
 	parent_class->start_book_view         = e_book_backend_ews_start_book_view;
 	parent_class->stop_book_view          = e_book_backend_ews_stop_book_view;
 
 	object_class->dispose                 = e_book_backend_ews_dispose;
 }
 
+static void
+e_book_backend_ews_authenticator_init (ESourceAuthenticatorInterface *interface)
+{
+	interface->try_password_sync = book_backend_ews_try_password_sync;
+}
+
 static void
 e_book_backend_ews_init (EBookBackendEws *backend)
 {
