@@ -32,10 +32,8 @@
 #include <glib/gstdio.h>
 #include <gio/gio.h>
 
-#include <libecal/e-cal-recur.h>
-#include <libecal/e-cal-time-util.h>
+#include <libecal/libecal.h>
 #include <libsoup/soup-misc.h>
-#include <libedataserver/e-source-list.h>
 
 #include "server/e-ews-connection.h"
 #include "server/e-ews-message.h"
