@@ -26,7 +26,8 @@
 
 #include <camel/camel.h>
 
-#include <e-ews-connection.h>
+#include "server/e-ews-connection.h"
+
 #include "camel-ews-store-summary.h"
 
 /* Standard GObject macros */
@@ -75,9 +76,6 @@ EEwsConnection *
 
 gboolean	camel_ews_store_connected	(CamelEwsStore *store,
 						 GError **error);
-void		camel_ews_store_maybe_disconnect
-						(CamelEwsStore *store,
-						 const GError *error);
 
 G_END_DECLS
 
