@@ -26,8 +26,10 @@
 #include <ctype.h>
 #include <glib.h>
 #include <glib/gprintf.h>
+
+#include "server/e-ews-connection.h"
+
 #include "utils.h"
-#include <e-ews-connection.h>
 
 void autocompletion_tests_run ();
 
@@ -56,7 +58,9 @@ resolve_names_cb (GObject *object,
 			EwsMailbox *mb = (EwsMailbox *) l->data;
 			g_print ("%s:%s \n", mb->name, mb->email);
 
-			e_ews_mailbox_free (mb);
+			g_free (mb->name);
+			g_free (mb->email);
+			g_free (mb);
 		}
 
 		g_slist_free (mailboxes);
@@ -86,11 +90,12 @@ op_test_resolve_names ()
 	g_assert_cmpstr (password, !=, NULL);
 	g_assert_cmpstr (uri, !=, NULL);
 
-	cnc = e_ews_connection_new (uri, username, password, TRUE, NULL, NULL, NULL);
+	cnc = e_ews_connection_new (uri, username, password, NULL, NULL, NULL);
 	g_assert (cnc != NULL);
-	e_ews_connection_resolve_names_start	(cnc, EWS_PRIORITY_MEDIUM, username,
-						 EWS_SEARCH_AD, NULL, FALSE, resolve_names_cb,
-						 cancellable, NULL);
+	e_ews_connection_resolve_names (
+		cnc, EWS_PRIORITY_MEDIUM, username,
+		EWS_SEARCH_AD, NULL, FALSE, cancellable,
+		resolve_names_cb, NULL);
 
 }
 
