@@ -26,10 +26,12 @@
 #include <ctype.h>
 #include <glib.h>
 #include <glib/gprintf.h>
+
+#include "server/e-ews-connection.h"
+#include "server/e-ews-item.h"
+#include "server/e-ews-message.h"
+
 #include "utils.h"
-#include <e-ews-connection.h>
-#include <e-ews-item.h>
-#include <e-ews-message.h>
 
 static void op_test_get_attachments ();
 void get_attachments_tests_run ();
@@ -79,7 +81,7 @@ op_test_get_attachments ()
 	g_assert_cmpstr (password, !=, NULL);
 	g_assert_cmpstr (uri, !=, NULL);
 
-	cnc = e_ews_connection_new (uri, username, password, TRUE, NULL, NULL, NULL);
+	cnc = e_ews_connection_new (uri, username, password, NULL, NULL, NULL);
 	g_assert (cnc != NULL);
 
 	attachmentid = "AAASAG1hbmR5Lnd1QGludGVsLmNvbQBGAAAAAACdSXexmsgJTpd3WpdX6ulXBwAm9E+BClHfQqEnvCoGvhheAAAAjpb6AACIeDU1D80fTrC3245yXdhOADUAPRB8AAABEgAQAIlh9YZzdzdMtvWW9ZI7+vM=";
@@ -89,11 +91,11 @@ op_test_get_attachments ()
 
 	tmpdir = g_build_filename ("/home/xwu1/evo-cache", NULL); //a test directory that exists already
 
-	e_ews_connection_get_attachments_start	(cnc, EWS_PRIORITY_MEDIUM,
-						 "xuidx", ids, tmpdir, TRUE,
-						 get_attachments_ready_callback,
-						 NULL, NULL,
-						 cancellable, NULL);
+	e_ews_connection_get_attachments (
+		cnc, EWS_PRIORITY_MEDIUM,
+		"xuidx", ids, tmpdir, TRUE,
+		NULL, NULL, cancellable,
+		get_attachments_ready_callback, NULL);
 	g_free (tmpdir);
 }
 
